unit ConfigUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, ExtDlgs, RomUnit;

type
  TConfig = class(TForm)
    PageControl: TPageControl;
    ConfigMessage: TTabSheet;
    ConfigFond: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    MemoMessage: TMemo;
    Label3: TLabel;
    PseudoTransparence: TRadioButton;
    Couleur: TRadioButton;
    Image: TRadioButton;
    PanelCouleur: TPanel;
    ImagePath: TEdit;
    ParcourirImage: TButton;
    Label4: TLabel;
    ColorDialog: TColorDialog;
    FontDialog: TFontDialog;
    OpenPictureDialog: TOpenPictureDialog;
    ImageCentree: TCheckBox;
    Label5: TLabel;
    Police: TTabSheet;
    ChangerPolice: TButton;
    Label6: TLabel;
    PanelCouleurClignotement: TPanel;
    Label7: TLabel;
    ShowCursor: TCheckBox;
    ConfigDivers: TTabSheet;
    UseForce: TCheckBox;
    Label8: TLabel;
    procedure ParcourirImageClick(Sender: TObject);
    procedure PanelCouleurClick(Sender: TObject);
    procedure ChangerPoliceClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    { Dclarations prives }
  public
    ColorTable: array[0..99] of TColor;
    function GetTexteFormat: string;
    procedure MajColorTable;
    procedure SaveToFile(FileName: string);
    procedure LoadFromFile(FileName: string);
  end;

var
  Config: TConfig;
  IntensTable: array[0..99] of double;

implementation

{$R *.dfm}

procedure WriteFont(Font: TFont; M: TExtMemoryStream);
begin
  M.WriteStr(Font.Name);
  M.WriteInteger(Font.Size);
  M.WriteBool(fsBold in Font.Style);
  M.WriteBool(fsItalic in Font.Style);
  M.WriteBool(fsUnderline in Font.Style);
  M.WriteBool(fsStrikeOut in Font.Style);
  M.WriteCardinal(Font.Color);
end;

procedure ReadFont(Font: TFont; M: TExtMemoryStream);
begin
  Font.Name := M.ReadStr;
  Font.Size := M.ReadInteger;
  Font.Style := [];
  if M.ReadBool then Font.Style := Font.Style + [fsBold];
  if M.ReadBool then Font.Style := Font.Style + [fsItalic];
  if M.ReadBool then Font.Style := Font.Style + [fsUnderline];
  if M.ReadBool then Font.Style := Font.Style + [fsStrikeOut];
  Font.Color := M.ReadCardinal;
end;

procedure WriteMemo(Memo: TMemo; M: TExtMemoryStream);
var i: integer;
begin
  M.WriteInteger(Memo.Lines.Count);
  for i := 0 to Memo.Lines.Count - 1 do
    M.WriteStr(Memo.Lines[i]);
end;

procedure ReadMemo(Memo: TMemo; M: TExtMemoryStream);
var i: integer;
    cnt: integer;
begin
  Memo.Lines.Clear;
  M.ReadInteger(cnt);
  for i := 0 to cnt - 1 do
    Memo.Lines.Add(M.ReadStr);
end;

{ TConfig }

function TConfig.GetTexteFormat: string;
var i: integer;
begin
  result := '';
  for i := 0 to MemoMessage.Lines.Count - 1 do
  begin
    if i = 0 then
      result := result + MemoMessage.Lines[i]
    else
      result := result + '|' + MemoMessage.Lines[i];
  end;
end;

procedure TConfig.ParcourirImageClick(Sender: TObject);
begin
  if OpenPictureDialog.Execute then
  begin
    ImagePath.Text := OpenPictureDialog.FileName;
    Image.Checked := true;
  end;
end;

procedure TConfig.PanelCouleurClick(Sender: TObject);
begin
  if ColorDialog.Execute then
  begin
    TPanel(Sender).Color := ColorDialog.Color;
    if Sender = PanelCouleur then Couleur.Checked := true;
  end;
end;

procedure TConfig.ChangerPoliceClick(Sender: TObject);
begin
  FontDialog.Execute;
end;

procedure TConfig.MajColorTable;
var i: integer;
    R1, G1, B1, R2, G2, B2: byte;
begin
  R1 := GetRValue(FontDialog.Font.Color);
  G1 := GetGValue(FontDialog.Font.Color);
  B1 := GetBValue(FontDialog.Font.Color);
  R2 := GetRValue(PanelCouleurClignotement.Color);
  G2 := GetGValue(PanelCouleurClignotement.Color);
  B2 := GetBValue(PanelCouleurClignotement.Color);
  for i := 0 to 99 do
  begin
    ColorTable[i] := rgb(
      R1 + trunc((R2 - R1) * IntensTable[i]),
      G1 + trunc((G2 - G1) * IntensTable[i]),
      B1 + trunc((B2 - B1) * IntensTable[i])
      );
  end;
end;

var i: integer;

procedure TConfig.LoadFromFile(FileName: string);
var M: TExtMemoryStream;
begin
  if not FileExists(FileName) then exit;

  M := TExtMemoryStream.Create;
  M.LoadFromFile(FileName);

  if M.ReadStrWOSize(4) = 'CDCS' then
  begin
    if M.ReadInteger = 1 then
    begin
      // Message
      ReadMemo(MemoMessage, M);
      // Police
      ReadFont(FontDialog.Font, M);
      PanelCouleurClignotement.Color := M.ReadCardinal;
      // Fond
      case M.ReadByte of
        1: Couleur.Checked := true;
        2: Image.Checked := true;
      else
        PseudoTransparence.Checked := true;
      end;
      PanelCouleur.Color := M.ReadCardinal;
      ImagePath.Text := M.ReadStr;
      ImageCentree.Checked := M.ReadBool;
      ShowCursor.Checked := M.ReadBool;
      // Divers
      UseForce.Checked := M.ReadBool;
    end;
  end;

  M.Free;
end;

procedure TConfig.SaveToFile(FileName: string);
var M: TExtMemoryStream;
begin
  M := TExtMemoryStream.Create;
  M.WriteStrWOSize('CDCS');
  M.WriteInteger(1);

  // Message
  WriteMemo(MemoMessage, M);
  // Police
  WriteFont(FontDialog.Font, M);
  M.WriteCardinal(PanelCouleurClignotement.Color);
  // Fond
  if Couleur.Checked then M.WriteByte(1) else
  if Image.Checked then M.WriteByte(2) else
    M.WriteByte(0);
  M.WriteCardinal(PanelCouleur.Color);
  M.WriteStr(ImagePath.Text);
  M.WriteBool(ImageCentree.Checked);
  M.WriteBool(ShowCursor.Checked);
  // Divers
  M.WriteBool(UseForce.Checked);

  // Fin
  M.SaveToFile(FileName);
  M.Free;
end;

procedure TConfig.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveToFile(GetExePath+'countdown.dat');
end;

procedure TConfig.FormCreate(Sender: TObject);
begin
  LoadFromFile(GetExePath+'countdown.dat');
end;

initialization
  // Cration de la table des intensits (formule totalement empirique ^^)
  for i := 0 to 99 do
    IntensTable[i] := exp( (i - 99) / 10);
end.
